<?php

namespace Drupal\mail_box_management\Plugin;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\node\Entity\Node;

/**
 * EntitySaver handles the saving of mail nodes.
 *
 * @file
 * EntitySaver.php contain EntitySaver class.
 */

/**
 * EntitySaver handles the saving of mail nodes.
 *
 * @class
 * EntitySaver handles the saving of mail nodes.
 */
class EntitySaver {

  /**
   * Entities data to be saved.
   *
   * @var array
   */
  private array $entities = [];

  /**
   * Add entity data to be saved.
   *
   * @param array $entities
   *   Data for new entities.
   * @param int $owner_uid
   *   Entity owner id.
   *
   * @return void
   *   nothing is returned.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function add(array $entities, int $owner_uid = 1): void {

    if (array_key_exists(0, $entities)) {

      foreach ($entities as $key => $entity) {
        if (!empty($entity['header']['Subject'])) {

          $attachments = [];
          if (!empty($entity['attachments'])) {
            foreach ($entity['attachments'] as $attachment) {
              $file_type = strtolower($attachment['type']);
              $stored_path = $this->write($file_type, base64_decode($attachment['content']));
              $file = File::create([
                'uri' => $stored_path,
              ]);
              $file->setOwnerId($owner_uid);
              $file->setPermanent();
              $file->save();
              $attachments[] = $file;
            }
          }

          $this->entities[] = [
            'field_email_subject' => $entity['header']['Subject'],
            'field_email_from_address' => $entity['header']['from_email'] ?? '',
            'field_email_from_name' => $entity['header']['from_name'] ?? '',
            'field_email_to_address' => $entity['header']['to_email'] ?? '',
            'field_email_to_name' => $entity['header']['to_name'] ?? '',
            'field_email_content' => $entity['body']['content'] ?? '',
            'field_email_attachments' => $attachments,
            'type' => 'mail_box_management_contents',
            'field_owner' => ['target_id' => $owner_uid],
            'field_message_id' => $entity['header']['message_id'],
          ];
        }
      }

    }

    else {

      if (!empty($entities['header']['Subject'])) {
        $entity = $entities;
        $attachments = [];
        if (!empty($entity['attachments'])) {
          foreach ($entity['attachments'] as $key => $attachment) {
            $file_type = strtolower($attachment['type']);
            $stored_path = $this->write($file_type, base64_decode($attachment['content']));
            $file = File::create([
              'uri' => $stored_path,
            ]);
            $file->setOwnerId($owner_uid);
            $file->setPermanent();
            $file->save();
            $attachments[] = $file;
          }
        }

        $this->entities[] = [
          'field_email_subject' => $entity['header']['Subject'],
          'field_email_from_address' => $entity['header']['from_email'] ?? '',
          'field_email_from_name' => $entity['header']['from_name'] ?? '',
          'field_email_to_address' => $entity['header']['to_email'] ?? '',
          'field_email_to_name' => $entity['header']['to_name'] ?? '',
          'field_email_content' => $entity['body']['content'] ?? '',
          'field_email_attachments' => $attachments,
          'type' => 'mail_box_management_contents',
          'field_owner' => ['target_id' => $owner_uid],
          'field_message_id' => $entity['header']['message_id'],
        ];
      }

    }

  }

  /**
   * Save the entities data added.
   *
   * @param string $namespace
   *   Mailbox name space.
   * @param int $uid
   *   Owner uid.
   *
   * @return bool
   *   True if node were created.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function save(string $namespace, int $uid): bool {
    $config = mail_box_management_service('mail_box_management.config');
    $imap_server_host = $config->getByOwnerId('host', $uid);

    if (!empty($imap_server_host)) {

      $node = mail_box_management_load($namespace, $imap_server_host, $uid);
      $name = $this->mailBoxNameFromNamespace($namespace);
      $name = $config->getByOwnerId('connection_type_title', $uid) . ' ' . $name;

      if (empty($node)) {

        $node = Node::create([
          'field_mail_box_imap_host' => $imap_server_host,
          'field_mail_box_name' => $namespace,
          'title' => strtolower($name),
          'type' => 'mail_box_content',
          'field_mailbox_owner' => ['target_id' => $uid],
        ]);
        $node->enforceIsNew();
        $node->save();

      }

      if ($node->id()) {

        $paragraphs = $node->get('field_mail_box_contents')?->getValue() ?? [];
        foreach ($this->entities as $entity) {

          $entity['field_mail_belongs_namespace'] = $namespace;
          if (empty(mail_box_management_paragraph($entity['field_message_id'], $namespace))) {
            $paragraph = mail_box_management_service('entity_type.manager')
              ->getStorage('paragraph')->create($entity);
            $paragraph->enforceIsNew();
            $paragraph->save();
            $paragraphs[] = $paragraph;
          }
        }
        $node->set('field_mail_box_contents', $paragraphs);
        return $node->save();
      }

    }
    return FALSE;
  }

  /**
   * Get Mailbox name.
   *
   * @param string $mailbox_namespace
   *   Namespace of mailbox.
   *
   * @return string
   *   Mailbox name is returned.
   */
  private function mailBoxNameFromNamespace(string $mailbox_namespace): string {
    $list = explode('}', $mailbox_namespace);
    $mailbox_name = end($list);
    if (str_contains($mailbox_name, '.')) {
      $list = explode('.', $mailbox_name);
      $mailbox_name = end($list);
    }
    if (str_contains($mailbox_name, '/')) {
      $list = explode('/', $mailbox_name);
      $mailbox_name = end($list);
    }
    return $mailbox_name;
  }

  /**
   * Files writer.
   *
   * @param string $file_type
   *   File type to be created.
   * @param string $data
   *   Data for the file.
   *
   * @return string|null
   *   Path to the file is return or null if failed.
   */
  private function write(string $file_type, string $data): ?string {
    $directory = "public://mailbox_management/attachments";
    $file_system = mail_box_management_service('file_system');
    try {
      if ($file_system->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY)) {
        $storage_path = $directory . '/attachment-' . time() . '.' . $file_type;
        $file_system->saveData($data, $storage_path);
        return $storage_path;
      }
    }
    catch (\Exception) {
    }
    return NULL;
  }

  /**
   * Static instance this class.
   *
   * @param array $data
   *   Entity data to create node.
   * @param int $owner_id
   *   Owner id.
   *
   * @return \Drupal\mail_box_management\Plugin\EntitySaver
   *   This object is return.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function create(array $data, int $owner_id): EntitySaver {
    $new = new static();
    $new->add($data, $owner_id);
    return $new;
  }

}
